package com.ningdatech.carapi.report.controller;

import java.math.BigDecimal;

import javax.validation.Valid;

import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.ningdatech.basic.model.PageVo;
import com.ningdatech.carapi.report.manage.BulkCementTransferBulletinManage;
import com.ningdatech.carapi.report.model.po.ReqBulkCementTransferDataUploadPO;
import com.ningdatech.carapi.report.model.po.ReqDataUploadListPO;
import com.ningdatech.carapi.report.model.vo.ResBulkCementTransferDataUploadDetailVO;
import com.ningdatech.carapi.report.model.vo.ResDataUploadListVO;
import com.ningdatech.log.annotation.WebLog;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;

/**
 * <p>
 * 散装水泥直报系统 前端控制器
 * </p>
 *
 * @author CMM
 * @since 2023-09-27
 */
@RequestMapping("/api/v1/direct/report/system")
@RestController
@Api(tags = "数据上报-中转库进出量统计")
@Validated
@RequiredArgsConstructor
public class BulkCementTransferBulletinController {

    private final BulkCementTransferBulletinManage bulkCementTransferBulletinManage;

    @PostMapping("/get/bulk-cement-transfer/last-month/data")
    @ApiOperation("获取中转库进出量上月数据")
    public BigDecimal getBulkCementTransferLastMonthData(@Valid @RequestBody ReqBulkCementTransferDataUploadPO po) {
        return bulkCementTransferBulletinManage.getBulkCementTransferLastMonthData(po);
    }

    @PostMapping("/get/bulk-cement-transfer/upload/data")
    @ApiOperation("获取中转库进出量计算数据")
    public ResBulkCementTransferDataUploadDetailVO getBulkCementSupplyUploadData(@Valid @RequestBody ReqBulkCementTransferDataUploadPO po) {
        return bulkCementTransferBulletinManage.getBulkCementTransferUploadData(po);
    }
    @PostMapping("/bulk-cement-transfer/data/upload")
    @ApiOperation("中转库进出量数据上报")
    @WebLog(value = "中转库进出量数据上报", modular = "数据上报-中转库进出量数据上报")
    public String bulkCementSupplyDataUpload(@Valid @RequestBody ReqBulkCementTransferDataUploadPO po) {
        return bulkCementTransferBulletinManage.bulkCementTransferDataUpload(po);
    }
    @PostMapping("/bulk-cement-transfer/data/upload/list")
    @ApiOperation("中转库进出量数据上报列表")
    public PageVo<ResDataUploadListVO> dataUploadList(@Valid @RequestBody ReqDataUploadListPO po) {
        return bulkCementTransferBulletinManage.dataUploadList(po);
    }
}
